;==============================================================
; Functions for placing the movable ships
;==============================================================
; Reads the joypad and moves/places the ship if a button is pressed
; Variables: ship_pos_x, ship_pos_y, ship_length
; Affects: a, bc
PlacePlayer:

    ;Place the ship at the start position
    ld a, StartPlayerX     ;32 pixels = 4 tiles in
    ld (ship_pos_x), a
    ld a, StartPlayerY     ;31 + 1 pixels = 4 tiles down
    ld (ship_pos_y), a

    ;Draw the ship
    call DrawShipSprites

PlaceShip:
    ;Keep reading the joypad
    in a,(Joypad)
    bit 0, a
    jp z, MoveUp
    bit 1, a
    jp z, MoveDown
    bit 2, a
    jp z, MoveLeft
    bit 3, a
    jp z, MoveRight
    bit 4, a
    jp z, Rotate
    bit 5, a
    jp z, Place
    jr PlaceShip

    ;==============================================================
    ; The "up" button is pressed
    ;==============================================================
MoveUp:
    ld a,(ship_pos_y)
    cp StartPlayerY       ;Check whether the ship has reached the edge of the board
    jp z, PlaceShip
    sub 8
    ld (ship_pos_y), a    ;Set the y coordinate to 1 tile less
    
    call DrawShipSprites  ;Redraw the sprites
-:  in a,(Joypad)         ;Wait for the button to be released again
    bit 0, a
    jr z,-
    jp PlaceShip

    ;==============================================================
    ; The "down" button is pressed
    ;==============================================================
MoveDown:
    ld c, MaxPlayerY      ;Bottom edge of the playing board
    ld a,(rotated)        ;Check whether the ship has been rotated
    bit 0, a
    jr z,+

    call MaxCoordinate    ;If the ship has been rotated

+:  ld a,(ship_pos_y)
    cp c                  ;Check whether the ship has reached the edge of the board
    jp z, PlaceShip
    add a, 8
    ld (ship_pos_y), a    ;Set the y coordinate to 1 tile more
    
    call DrawShipSprites  ;Redraw the sprites
-:  in a,(Joypad)         ;Wait for the button to be released again
    bit 1, a
    jr z,-
    jp PlaceShip

    ;==============================================================
    ; The "left" button is pressed
    ;==============================================================
MoveLeft:
    ld a,(ship_pos_x)
    cp StartPlayerX       ;Check whether the ship has reached the edge of the board
    jp z, PlaceShip
    sub 8
    ld (ship_pos_x), a    ;Set the x coordinate to 1 tile less

    call DrawShipSprites  ;Redraw the sprites
-:  in a,(Joypad)         ;Wait for the button to be released again
    bit 2, a
    jr z,-
    jp PlaceShip

    ;==============================================================
    ; The "right" button is pressed
    ;==============================================================
MoveRight:
    ld c, MaxPlayerX      ;Right edge of the playing board
    ld a,(rotated)        ;Check whether the ship has been rotated
    bit 0, a
    jr nz,+

    call MaxCoordinate    ;If the ship has not been rotated

+:  ld a,(ship_pos_x)
    cp c                  ;Check whether the ship has reached the edge of the board
    jp z, PlaceShip
    add a, 8
    ld (ship_pos_x), a    ;Set the x coordinate to 1 tile more
    
    call DrawShipSprites  ;Redraw the sprites
-:  in a,(Joypad)         ;Wait for the button to be released again
    bit 3, a
    jr z,-
    jp PlaceShip

    ;==============================================================
    ; The "1" button is pressed
    ;==============================================================
Rotate:
    ld a,(rotated)        ;Rotate/unrotate the ship (toggle "rotated")
    xor 1
    ld (rotated), a

    bit 0, a              ;Check whether the ship is now rotated
    jr nz,+

    ;If it is rotated:
    ld c, MaxPlayerX      ;Right edge of the playing board
    call MaxCoordinate    ;Find the max cordinate
    ld a,(ship_pos_x)
    ld b, a
    ld a, c               ;Test whether the ship is now violating the border..
    sub b
    jp p, DoRotate
    ld a, c               ;..and if it does, set the x cordinate to the max value
    ld (ship_pos_x), a
    jr DoRotate

    ;If it is not rotated:
+:  ld c, MaxPlayerY      ;Bottom edge of the playing board
    call MaxCoordinate    ;Find the max cordinate
    ld a,(ship_pos_y)
    ld b, a
    ld a, c               ;Test whether the ship is now violating the border..
    sub b
    jp p, DoRotate
    ld a, c               ;..and if it does, set the y cordinate to the max value
    ld (ship_pos_y), a

    DoRotate:
    call DrawShipSprites  ;Redraw the sprites
-:  in a,(Joypad)         ;Wait for the button to be released again
    bit 4, a
    jr z,-
    jp PlaceShip

    ;==============================================================
    ; The "2" button is pressed
    ;==============================================================
Place:
    ; 1. Make sure, that there is not already a ship in that place
    call SpriteToBoard
    
    ld a,(ship_length)
    ld b, a
    ld a,(rotated)        ;Check whether the ship was rotated
    bit 0, a
    jr nz, BoardRotated


    ;If the ship was not rotated:
-:  ld a, (hl)            ;Look for a ship "ship_length" to the right
    bit 6, a
    jr z,+
    call WaitForIt
    call DenySound
    jp PlaceShip
+:  inc hl
    djnz -
    jr PlaceBoard
    
    ;If the ship was rotated:
    BoardRotated:
    ld de, 8
-:  ld a, (hl)            ;Look for a ship "ship_length" down
    bit 6, a
    jr z,+
    call WaitForIt
    call DenySound
    jp PlaceShip
+:  add hl, de
    djnz -

    PlaceBoard:
    ; 2. Place the ship on the "invisible" player board
    call SpriteToBoard

    ld a,(ship_length)
    ld b, a
    ld a,(rotated)        ;Check whether the ship was rotated
    bit 0, a
    jr nz,+

    ;If the ship was not rotated:
    ld a,(ship_type)
-:  ld (hl), a
    inc hl
    djnz -
    jr PlaceTiles

    ;If the ship was rotated:
+:  ld a,(ship_type)
    ld de, 8
-:  ld (hl), a
    add hl, de
    djnz -

    PlaceTiles:
    ; 3. Find the location of the first sprite on the tilemap

    ld a,(ship_pos_x)     ;ship_pos_x / 4
    ld e, 0
-:  inc e
    sub 4
    jr nz,-

    ld a,(ship_pos_y)     ;(ship_pos_y + 1)* 8
    inc a
    ld c, a
    ld b, 0
    ld d, 8
    ld hl, 0
-:  add hl, bc
    dec d
    jr nz,-
    ld d, 0

    add hl, de            ;add the two numbers to get the background offset
    ld b, h
    ld c, l

    ; 4. Set VRAM write address to needed tilemap index
    ld hl,$3800 | VRAMWrite
    add hl, bc
    call SetVDPAddress

    ; 5. Place tiles at the same locations as the sprites
    ld a,(ship_length)
    ld b, a
    ld de, 64
-:  ld a,(ship_tile_nr)
    out (VDPData), a
    xor a
    out (VDPData), a
    ld a, (rotated)       ;Check whether the ship is rotated
    bit 0, a
    jr z,+
    add hl, de            ;If it is rotated, add 64 to the tilemap index for each tile
    call SetVDPAddress
+:  djnz -

    call RemoveSprites
    call MissSound

    WaitForIt:
-:  in a,(Joypad)         ;Wait for the button to be released again
    bit 5, a
    jr z,-
    ret
    
;==============================================================
; Helper functions
;==============================================================
DrawShipSprites:
; Draws the sprites that represent a movable ship
; A non rotated ship is drawn from left to right
; A rotated ship is drawn downwards
; Variables: ship_pos_x, ship_pos_y, ship_length, ship_tile_nr
; Affects: a, hl, bc

    ld a,(rotated)         ;Check whether the ship has been rotated
    bit 0, a
    jr nz,+

    ; If it has not been rotated:

    ; Set VRAM write address to start of x coordinates and tile indexes
    ld hl,$3F80 | VRAMWrite
    call SetVDPAddress

    ld a,(ship_length)
    ld b, a
    ld a,(ship_pos_x)
    ld c, a

-:  ld a, c               ;Set x coordinate
    out (VDPData), a
    add a, 8              ;Place the next sprite 1 tile to the right
    ld c, a
    ld a,(sprite_tile_nr) ;Set tile number
    out (VDPData), a
    djnz -                ;Draw "ship_length" number of sprites

    ; Set VRAM write address to start of y coordinates
    ld hl,$3F00 | VRAMWrite
    call SetVDPAddress
    
    ld a,(ship_length)
    ld b, a
    ld a,(ship_pos_y)
    
-:  out (VDPData), a      ;All sprites get the same y coordinate
    djnz -
    ld a, $D0
    out (VDPData), a
    ret

    ; If it has been rotated:

    ; Set VRAM write address to start of x coordinates and tile indexes
+:  ld hl,$3F80 | VRAMWrite
    call SetVDPAddress

    ld a,(ship_length)
    ld b, a
    ld a,(ship_pos_x)
    ld c, a
    
-:  ld a, c
    out (VDPData), a      ;All sprites get the same x coordinate
    ld a,(sprite_tile_nr) ;Set tile number
    out (VDPData), a
    djnz -
    
    ; Set VRAM write address to start of y coordinates
    ld hl,$3F00 | VRAMWrite
    call SetVDPAddress
    
+:  ld a,(ship_length)
    ld b, a
    ld a,(ship_pos_y)
    ld c, a

-:  ld a, c               ;Set y coordinate
    out (VDPData), a
    add a, 8              ;Place the next sprite 1 tile down
    ld c, a
    djnz -
    ld a, $D0
    out (VDPData), a
    ret
    
MaxCoordinate:
; Returns a max coordinate for the first sprite in a ship
; based on the edge coordinate and the ship length
; Parameters: c = edge of the playing board
; Returns: c = max coordinate for the first sprite
; Variables: ship_length
; Affects: a, bc
    ld a,(ship_length)
    ld b, a
    ld a, c
-:  sub 8                  ;Subtract "ship length" tiles from edge
    djnz -
    add a, 8               ;And add one tile
    ld c, a
    ret

SpriteToBoard:
; Finds the location of the first sprite on the playerboard
; Returns: hl = address corresponding to the first sprite location on the board
; Variables: ship_pos_x, ship_pos_y
; Affects: a, bc, hl
    ld a,(ship_pos_x)
    sub StartPlayerX      ;(ship_pos_x - StartPlayerX) / 8
    jr nz,+
    ld b, 0
    jr ZeroCase
+:  ld b, 0
-:  inc b
    sub 8
    jr nz,-

    ZeroCase:
    ld a,(ship_pos_y)     ; ship_pos_y - StartPlayerY
    sub StartPlayerY

    add a, b              ;add the two numbers to get the board offset
    ld c, a
    ld b, 0
    ld hl, player_board
    add hl, bc            ;First sprite location on the player board
    ret