PlayerStrike:
;==============================================================
; Functions for striking the enemy
;==============================================================
; Reads the joypad and moves/fires at the target
; Variables: crosshair_x, crosshair_y
; Affects: a, bc

    call DrawCrosshair

    in a,(Joypad)
    bit 0, a
    jp z, SweepUp
    bit 1, a
    jp z, SweepDown         ;Keep reading the joypad
    bit 2, a
    jp z, SweepLeft
    bit 3, a
    jp z, SweepRight
    bit 5, a
    jp z, StrikeAtEnemy
    jr PlayerStrike

    ;==============================================================
    ; The "up" button is pressed
    ;==============================================================
SweepUp:
    ld a,(crosshair_y)
    cp StartEnemyY         ;Check whether the crosshair has reached the edge of the board
    jp z, PlayerStrike
    sub 8
    ld (crosshair_y), a    ;Set the y coordinate to 1 tile less

-:  in a,(Joypad)          ;Wait for the button to be released again
    bit 0, a
    jr z,-
    call DrawCrosshair     ;Redraw the sprite
    jp PlayerStrike

    ;==============================================================
    ; The "down" button is pressed
    ;==============================================================
SweepDown:
    ld a,(crosshair_y)
    cp MaxEnemyY           ;Check whether the crosshair has reached the edge of the board
    jp z, PlayerStrike
    add a, 8
    ld (crosshair_y), a    ;Set the y coordinate to 1 tile more

-:  in a,(Joypad)          ;Wait for the button to be released again
    bit 1, a
    jr z,-
    call DrawCrosshair     ;Redraw the sprite
    jp PlayerStrike

    ;==============================================================
    ; The "left" button is pressed
    ;==============================================================
SweepLeft:
    ld a,(crosshair_x)
    cp StartEnemyX         ;Check whether the crosshair has reached the edge of the board
    jp z, PlayerStrike
    sub 8
    ld (crosshair_x), a    ;Set the y coordinate to 1 tile less

-:  in a,(Joypad)          ;Wait for the button to be released again
    bit 2, a
    jr z,-
    call DrawCrosshair     ;Redraw the sprite
    jp PlayerStrike

    ;==============================================================
    ; The "right" button is pressed
    ;==============================================================
SweepRight:
    ld a,(crosshair_x)
    cp MaxEnemyX           ;Check whether the crosshair has reached the edge of the board
    jp z, PlayerStrike
    add a, 8
    ld (crosshair_x), a    ;Set the y coordinate to 1 tile more

-:  in a,(Joypad)          ;Wait for the button to be released again
    bit 3, a
    jr z,-
    call DrawCrosshair     ;Redraw the sprite
    jp PlayerStrike

    ;==============================================================
    ; The "2" button is pressed
    ;==============================================================

StrikeAtEnemy:
    ;Find Crosshair location on the enemy board
    ld a,(crosshair_x)
    sub StartEnemyX      ;(crosshair_x - StartEnemyX) / 8
    jr nz,+
    ld b, 0
    jp ++
+:  ld b, 0
-:  inc b
    sub 8
    jr nz,-

++: ld a,(crosshair_y)     ; crosshair_y - StartEnemyY
    sub StartEnemyY

    add a, b              ;add the two numbers to get the board offset
    ld c, a
    ld b, 0
    ld hl, enemy_board
    add hl, bc            ;Crosshair location on the enemy board
    
    ;Check whether the tile has been hit before
    ld a,(hl)
    bit 7, a
    jr z,+
    call PlayerStrikeWait
    call DenySound
    jp PlayerStrike

    ;Redraw the tile and set the score:
    

+:  and %00111111              ;Remove status bits

    ;Look for the aircraft carrier
    cp 1
    jr nz,+                    ;If it's not the aircraft carrier, look for the next ship type
                               ;But if it is...
    call EnemyBoardOffset      ;Find the background offset
    call DrawHole
    call StrikeSound
    ld a,(enemy_air)           ;Take one "life" from the aircraft carrier
    dec a
    jr nz,++                   ;Test whether the aircraft carrier is out of lives
                               ;And if it is...
    call RemoveSprites
    call WaitAMoment
    ld hl,$392E | VRAMWrite
    call SetVDPAddress
    ld b, 5
-:  call DrawHole              ;Draw holes on the status aircraft carrier
    call StrikeSound
    djnz -

    ld a,(enemy_ships)         ;Take one "life" from the enemy
    dec a
    jr nz,+++                  ;Test whether the enemy is out of lives
                               ;And if he is...
    jp WinScreen                    ;End of the world

+++:ld (enemy_ships), a
    xor a
++: ld (enemy_air), a
    call PlayerStrikeWait      ;Wait for the button to be released
    jp PlayerStrike            ;Take another shot

    ;Repeat this with all the other ships...

    ;Look for the battleship
+:  cp 2
    jr nz,+
    call EnemyBoardOffset
    call DrawHole
    call StrikeSound
    ld a,(enemy_bat)
    dec a
    jr nz,++
    call RemoveSprites
    call WaitAMoment
    ld hl,$39B0 | VRAMWrite
    call SetVDPAddress
    ld b, 4
-:  call DrawHole
    call StrikeSound
    djnz -
    ld a,(enemy_ships)
    dec a
    jr nz,+++
    jp WinScreen
+++:ld (enemy_ships), a
    xor a
++: ld (enemy_bat), a
    call PlayerStrikeWait
    jp PlayerStrike

    ;Look for the cruiser
+:  cp 3
    jr nz,+
    call EnemyBoardOffset
    call DrawHole
    call StrikeSound
    ld a,(enemy_cru)
    dec a
    jr nz,++
    call RemoveSprites
    call WaitAMoment
    ld hl,$3A32| VRAMWrite
    call SetVDPAddress
    ld b, 3
-:  call DrawHole
    call StrikeSound
    djnz -
    ld a,(enemy_ships)
    dec a
    jr nz,+++
    jp WinScreen
+++:ld (enemy_ships), a
    xor a
++: ld (enemy_cru), a
    call PlayerStrikeWait
    jp PlayerStrike
    
    ;Look for the first destroyer
+:  cp 4
    jr nz,+
    call EnemyBoardOffset
    call DrawHole
    call StrikeSound
    ld a,(enemy_des_1)
    dec a
    jr nz,++
    call RemoveSprites
    call WaitAMoment
    ld hl,$3928| VRAMWrite
    call SetVDPAddress
    ld b, 2
-:  call DrawHole
    call StrikeSound
    djnz -
    ld a,(enemy_ships)
    dec a
    jr nz,+++
    jp WinScreen
+++:ld (enemy_ships), a
    xor a
++: ld (enemy_des_1), a
    call PlayerStrikeWait
    jp PlayerStrike
    
    ;Look for the second destroyer
+:  cp 5
    jr nz,+
    call EnemyBoardOffset
    call DrawHole
    call StrikeSound
    ld a,(enemy_des_2)
    dec a
    jr nz,++
    call RemoveSprites
    call WaitAMoment
    ld hl,$39A8| VRAMWrite
    call SetVDPAddress
    ld b, 2
-:  call DrawHole
    call StrikeSound
    djnz -
    ld a,(enemy_ships)
    dec a
    jr nz,+++
    jp WinScreen
+++:ld (enemy_ships), a
    xor a
++: ld (enemy_des_2), a
    call PlayerStrikeWait
    jp PlayerStrike

    ;Look for the first submarine
+:  cp 6
    jr nz,+
    call EnemyBoardOffset
    call DrawHole
    call StrikeSound
    ld a,(enemy_sub_1)
    dec a
    jr nz,++
    call RemoveSprites
    call WaitAMoment
    ld hl,$3A28| VRAMWrite
    call SetVDPAddress
    call DrawHole
    call StrikeSound
    ld a,(enemy_ships)
    dec a
    jr nz,+++
    jp WinScreen
+++:ld (enemy_ships), a
    xor a
++: ld (enemy_sub_1), a
    call PlayerStrikeWait
    jp PlayerStrike

    ;Look for the second submarine
+:  cp 7
    jr nz,+
    call EnemyBoardOffset
    call DrawHole
    call StrikeSound
    ld a,(enemy_sub_2)
    dec a
    jr nz,++
    call RemoveSprites
    call WaitAMoment
    ld hl,$3A2C| VRAMWrite
    call SetVDPAddress
    call DrawHole
    call StrikeSound
    ld a,(enemy_ships)
    dec a
    jr nz,+++
    jp WinScreen
+++:ld (enemy_ships), a
    xor a
++: ld (enemy_sub_2), a
    call PlayerStrikeWait
    jp PlayerStrike
    
    ;It's just water
+:  call EnemyBoardOffset
    ld a, $5B
    out (VDPData), a
    xor a
    out (VDPData), a
    call MissSound

    PlayerStrikeWait:
-:  in a,(Joypad)          ;Wait for the button to be released again
    bit 5, a
    jr z,-
    
    call RemoveSprites

    ret
 
;==============================================================
; Helper functions
;==============================================================

DrawCrosshair:
    ; Set VRAM write address to start of x coordinates and tile indexes
    ld hl,$3F80 | VRAMWrite
    call SetVDPAddress

    ld a,(crosshair_x)     ;Set x coordinate
    out (VDPData), a
    ld a, $68              ;Set tile number (Cross)
    out (VDPData), a

    ; Set VRAM write address to start of y coordinates
    ld hl,$3F00 | VRAMWrite
    call SetVDPAddress

    ld a,(crosshair_y)
    out (VDPData), a       ;Set y coordinate
    ld a, $D0
    out (VDPData), a
    ret
    
EnemyBoardOffset:
    ld a,(hl)              ;Mark the tile as hit
    or %10000000
    ld (hl), a

    ; Find crosshair location on the background
    ld a,(crosshair_x)     ;crosshair_x / 4
    ld e, 0
-:  inc e
    sub 4
    jr nz,-

    ld a,(crosshair_y)     ;(crosshair_y + 1)* 8
    inc a
    ld c, a
    ld b, 0
    ld d, 8
    ld hl, 0
-:  add hl, bc
    dec d
    jr nz,-
    ld d, 0

    add hl, de             ;add the two numbers to get the background offset
    ld b, h
    ld c, l

    ; Set VRAM write address to needed tilemap index
    ld hl,$3800 | VRAMWrite
    add hl, bc
    call SetVDPAddress
    ret
    
WinScreen:
    ; 1. Set VRAM write address to tilemap index 0
    ld hl,$3800 | VRAMWrite
    call SetVDPAddress

    ; 2. Output background data
    ld hl,GameOverBackgroundData
    ld bc,GameOverBackgroundDataEnd-GameOverBackgroundData  ; Counter for number of bytes to write
    call CopyToVDP
    
    ld hl,$3A60 | VRAMWrite
    call SetVDPAddress

    ;WIN!:
    ld a, $62
    out (VDPData), a
    xor a
    out (VDPData), a
    ld a, $63
    out (VDPData), a
    xor a
    out (VDPData), a
    ld a, $64
    out (VDPData), a
    xor a
    out (VDPData), a
    ld a, $17
    out (VDPData), a
    xor a
    out (VDPData), a
    
-:  in a,(Joypad)         ;Wait for the start button (1) to be pressed
    bit 4, a
    jp nz,-
    jp NewGame